# --- [ bargraph.py ] ---------------------------------------------

from tkinter import *
from string import *
import time
import _thread

class BarGraph(Canvas):
    """ Zeichnet einen Bargraphen der Breite 'width' und der Höhe 'heigth'

        die Ausrichtung wird orientation = ["ewns"] definiert
        Um den Bargraphen zu zeigen muss er mit draw() gezeichnet werden"""
        
    def __init__(self, master,
                 width = "200",                 # Ausdehnung in X-Richtung in Pixeln
                 heigth = "20",                 # Ausdehnung in y-Richtung in Pixeln
                 start_value ="0",              # der Anfangswert, Länge = 0
                 end_value = "200",             # der Endwert, max. Länge des Bargraphen
                 orientation = "e",             # die Ausrichtung ["ewns"]
                 bg_color  = "#D8D8D8",         # die Hintergrundfarbe
                 bar_color = "#B0B0C0",         # die Farbe des Balkens
                 font = ("Courier", 10, "bold"),# der Font der Beschriftung 
                 show_value = True,             # Beschriftung zeigen oder nicht
                 label = (1, "", ""),           # Faktor sowie Vor-/Nachlauftext für die Beschriftung
                 **kw):

        Canvas.__init__(self, master, width = width, height = heigth, **kw)
        self.__start_value = int(start_value)
        self.__end_value   = int(end_value)
        self.__pixel_x     = width
        self.__pixel_y     = heigth
        self.__orientation = orientation.lower()# Bargraphrichtung nach -> "ewns"
        if self.__orientation in ("ew"):        # horizontale Ausrichtung
            self.__length = int(self.__pixel_x) # die max. Länge des Bargraphen
        else:
            self.__length = int(self.__pixel_y)

        self.__bg_color    = bg_color
        self.__bar_color   = bar_color
        self.__show_value  = show_value
        self.__label       = label

        # die Hintergrundfläche ist unabhängig von der Richtung des Bargraphen
        self.__bargnd = self.create_rectangle("0","0", width, heigth, fill = bg_color)
        # der Bargraph wird als leere Fläche eingerichtet, die mit draw() aktualisiert werden kann
        self.__bar    = self.create_rectangle("0", "0", "0",  "0", fill = bar_color)

        if self.__show_value:
            self.__bartxt = self.create_text(str(int(width) // 2), str(int(heigth) // 2), text = "0", font = font)

# -----------------------
    def draw(self, value):
        
#        if type(value) == str:
#            value = int(value)

        if value < self.__start_value:      # der Bargraph ist nicht sichtbar 
            bar_x = 0
                        
        elif value > self.__end_value:      # der Bargraph überdeckt den gesamten Bereich
            bar_x = self.__length

        else:                               # der Bargraph liegt irgendwo 
            bar_x = self.__end_value - self.__start_value
            bar_x = (value - self.__start_value) * self.__length // bar_x

        
        # die Orientierung des Bargraphen muss hier unterschieden werden
        
        if self.__orientation == 'e':       # horizontaler Bargraph von links nach rechts
            self.coords(self.__bar, "0", "0", str(bar_x), self.__pixel_y)

        elif self.__orientation == 'w':     # horizontaler Bargraph von rechts nach links
            self.coords(self.__bar, str(self.__length - bar_x), "0", self.__pixel_x, self.__pixel_y)


        elif self.__orientation == 's':     # vertikaler Bargraph von oben nach unten
            self.coords(self.__bar, "0", "0", self.__pixel_x, str(bar_x))

        elif self.__orientation == 'n':     # vertikaler Bargraph von unten nach oben
            self.coords(self.__bar, "0", self.__pixel_y, self.__pixel_x, str(self.__length - bar_x),)
            
        if self.__show_value:               # ggf. in der Balkenmitte den aktuellen Wert ausgeben
                self.itemconfigure(self.__bartxt, text = self.__label[1] + str(int(value * self.__label[0])) + self.__label[2])

            

if __name__ == '__main__':

# -----------------------------------
    def animate():
        v = -10
        while True:
            time.sleep(.5)
            bg1.draw(v)
            bg2.draw(v)
            bg3.draw(v)
            bg4.draw(v)
            v += 10
            if v > 210:
                v = -10

# ------------------------------------
    top = Tk()
    top.title("GUI-Test")

    frm1 = LabelFrame(top, text = "Bargraph-Test", height = 300)
    frm1.grid(column = 0, row = 0, padx = 5, pady = 5)

    bg1  = BarGraph(frm1, start_value = -10, width = 50, bar_color = "red", bg_color = "green")
    bg1.grid(column = 0, row = 0, padx = 10, pady = 10, sticky = "w")

    bg2  = BarGraph(frm1, start_value = -10, orientation = "w", label = (1, "Vor ", " Nach"), bar_color = "#FFA0A0")
    bg2.grid(column = 0, row = 1, padx = 10, pady = 10)

    bg3  = BarGraph(frm1, start_value = 0, end_value = 300, width = 30, heigth = 100, orientation = "s")
    bg3.grid(column = 1, row = 0, padx = 10, pady = 10, rowspan = 2)

    bg4  = BarGraph(frm1, start_value = -10, width = 30, heigth = 100, orientation = "n", bar_color = "#A0A0FF")
    bg4.grid(column = 2, row = 0, padx = 10, pady = 10, rowspan = 2)
    
    _thread.start_new_thread(animate,())

    top.mainloop()
